"""
    Title: HeadKuarter
    Module Name: creatContainer
    Language: Python
    Date Created: 
    Date Modified: 7-05-2022
    Description:
        ###############################################################
        ##                    Password Genrator                      ## 
        ###############################################################
 """

import random
import sys
from . import mnemonic


class Generator:
    def __init__(self, entropy_size: int, chosen_theme: str, entropy):
        # Pick "finances" as default, if no theme is given
        chosen_theme = (chosen_theme if chosen_theme is not None else "finances")
        self.m = mnemonic.Mnemonic(chosen_theme)
        self.phrase_len = len(self.m.words_dictionary["FILLING_ORDER"])
        # Pick random if the entropy input is none
        entropy_bits = (entropy if entropy is not None else bytearray(
            [random.getrandbits(8) for _ in range(entropy_size//8)]))
        self.phrase = self.m.to_mnemonic(entropy_bits).split()

    def show_phrases(self) -> list[str]:
        """
            This method prints the generated phrases and returns the phrases
        Returns
        -------
        str
            This is the returned phrases generated as a string variable
        """
        '''  for i in range(len(self.phrase)):
            print(self.phrase[i][0:2], end="")
        
        for i in range(len(self.phrase)//self.phrase_len):
            print(" ".join(self.phrase[self.phrase_len*i:self.phrase_len*(i+1)]))'''
        return self.phrase








import argparse
import hvac
import os

def main():
    """
    The main program entry-point.
    """

    parser = argparse.ArgumentParser(__file__, __doc__,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument("--vault-address",
        default=os.getenv("VAULT_ADDR"),
        help="The address of the Vault server."
    )
    parser.add_argument("--vault-token",
        default=os.getenv("VAULT_TOKEN"),
        help="The token for authentication to the Vault server."
    )
    parser.add_argument("--vault-path",
        required=True,
        help="The path of the target secret in Vault."
    )
    args = parser.parse_args()

    if not args.vault_address:
        parser.exit(status=1, message="Must specify address of Vault server using --vault-address argument or VAULT_ADDR environment variable.")

    if not args.vault_token:
        parser.exit(status=1, message="Must specify security token for Vault server using --vault-token argument or VAULT_TOKEN environment variable.")

    client = hvac.Client(args.vault_address, token=args.vault_token)
    secret = client.read(args.vault_path)
    if secret is None:
        parser.exit(status=2, message="Secret not found at '{}/{}'.".format(args.vault_address, args.vault_path))

    secret_data = secret["data"]
    for name, value in secret_data.items():
        # Escape symbols commonly used by Bash.
        value = value.replace('"', '\\"').replace('$', '\\$').replace('`', '\\`')

        print('export TF_VAR_{}="{}"'.format(
            name,
            value
        ))

if __name__ == '__main__':
    main()



import sys
import urllib,os,subprocess
import time
import urllib.request
import requests
plt = sys.platform

def win():
    import ctypes, sys
    def is_admin():
        try:
            return ctypes.windll.shell32.IsUserAnAdmin()
        except:
            return False
    if is_admin():
        from pywinauto.application import Application
        from pywinauto.keyboard import send_keys
        url = 'https://launchpad.net/veracrypt/trunk/1.25.9/+download/VeraCrypt%20Setup%201.25.9.exe' 
        filename = 'VeraCrypt Setup 1.25.9.exe'  
        
        with open(filename, "wb") as f:
            print("Downloading %s" % filename)
            response = requests.get(url, stream=True)
            total_length = response.headers.get('content-length')
            if total_length is None: 
                f.write(response.content)
            else:
                dl = 0
                total_length = int(total_length)
                for data in response.iter_content(chunk_size=4096):
                    dl += len(data)
                    f.write(data)
                    done = int(50 * dl / total_length)
                    sys.stdout.write("\r[%s%s]" % ('=' * done, ' ' * (50-done)) )    
                    sys.stdout.flush()
        urllib.request.urlretrieve(url, filename)
        print(os.getcwd())
        app = Application().start(r'VeraCrypt Setup 1.25.9.exe')
        time.sleep(2)
        send_keys("{ENTER}")
        time.sleep(2)
        send_keys("{a}")
        send_keys("{ENTER}")
        send_keys("{ENTER}")
        send_keys("{ENTER}")
        send_keys("{ENTER}")
        send_keys("{ENTER}")
        send_keys("{ENTER}")
        send_keys("{LEFT}")
        send_keys("{ENTER}")
        send_keys("{LEFT}")
        send_keys("{ENTER}")
    else:
        # Re-run the program with admin rights
        ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, " ".join(sys.argv), None, 1)
    

def mac():
    url = "https://launchpad.net/veracrypt/trunk/1.25.9/+download/VeraCrypt_1.25.9.dmg"
    filename = 'VeraCrypt Setup 1.25.9.exe'  
    urllib.request.urlretrieve(url, filename)

def lin():
    from pywinauto.linux.keyboard import send_keys
    send_keys('{ENTER}')
    subprocess.run(["sudo","add-apt-repository","ppa:unit193/encryption"],check=True)
    send_keys('^y')
    subprocess.run(["sudo","apt","update"],check=True)
    subprocess.run(["sudo","apt","install","veracrypt"],check=True)



"""
    Title: HeadKuarter
    Module Name: passwordGenratorLC
    Language: Python
    Date Created: 23-05-2022
    Date Modified: 23-05-2022
    Description:
        ###############################################################
        ##  genrate password ussing formosa                          ## 
        ###############################################################
 """

from . import generator
from . import mnemonic
from pathlib import Path
import copy
import random
#import swapper
import json
import gc
import traceback




def get_directory() -> Path:
        """
            This method finds out in which directory the code is running

        Returns
        -------
        path
            Returns the absolute path found of the file
        """
        return Path(__file__).parent.absolute()

# varible 
selected_theme = "finances"
directory_path = (get_directory() / "themes")
files_path = Path(directory_path)
# Finding paths and filtering .json files
themes = tuple([each_directory.stem for each_directory in files_path.glob(r"*.json")])
base_dict = mnemonic.Mnemonic(selected_theme).words_dictionary
edited_dict = copy.deepcopy(base_dict)

global last_text 
global last_n_phrases  
global password_lines
last_text = ""
password_lines = []
last_n_phrases = 0


var_check_number=False
var_check_case = False
var_check_char = False


number_phrases = 1 # initial value

def generate_text(selected_theme,number_phrases,rbn,rbc,sc,str_pwd):
        """
            This method calls object Generator which build a mnemonic phrase in formosa standard
            then updates displayed text
        """
        g = generator.Generator(number_phrases * 32, selected_theme, None)
       
        phrases = g.show_phrases()


        global last_text
        global last_n_phrases 
        global password_lines
        

        text = ""
        phrase_len = len(base_dict["NATURAL_ORDER"])
        for word_index in range(len(phrases)):
            text = text + phrases[word_index][0:2]
        password_genrated = text
        text = ""
        for phrase_index in range(len(phrases) // phrase_len):
            text = text + " ".join(phrases[phrase_len * phrase_index:phrase_len * (phrase_index + 1)]) + "\n"

       
        password_genrated = insert_number(password_genrated,rbn)
        password_genrated = insert_spc_char(password_genrated,rbc)
        password_genrated = insert_swap_case(password_genrated,sc)

        if (str_pwd=='file'):
            with open('password.txt','w') as f:
                f.write(password_genrated)  
            f.close()

            with open('m-phrasess.txt','w') as f:
                f.write(text)  
            f.close()
            print("Password is coppied to password.txt file!")
        elif(str_pwd == 'ram'):
            #code for enviorment variable creation
            pyperclip.copy(password_genrated)
            print("Password is copied to clipboard!")
        
        return(password_genrated)


def insert_number(text,flag):
        """
            This method changes an ordinary character to a number
        """
        to_replace = ["a", "e", "i", "o", "b", "g", "t"]
        replace_by = ["4", "3", "1", "0", "8", "6", "7"]
        text = insert_character(text,to_replace, replace_by, flag)
        return(text)

def insert_spc_char(text,flag):
        """
            This method changes an ordinary character to a special character
        """
        to_replace = ["c", "h", "l", "s", "j"]
        replace_by = ["¢", "#", "£", "$", "!"]
        return(insert_character(text,to_replace, replace_by, flag))

def insert_swap_case(text,flag):
        """
            This method swaps the case of a character
        """
        to_replace = list(map(chr, range(97, 123)))
        replace_by = [each_char.swapcase() for each_char in to_replace]
        return(insert_character(text,to_replace, replace_by, flag))

def insert_character(text,to_replace: list, replace_by: list, check_box_var: bool):
        """
            This method finds out if the text in the texbox is written as natural or edited
            and calls method replace_characters() with correct order of variables

        Parameters
        ----------
        to_replace : list
            The list of characters to be removed
        replace_by : list
            The list of characters to be inserted
        check_box_var : bool
            This variable tells if the word is written as natural, or it is already edited
        """
      
        #text = 'VGmkjrewqpobdcxt'
        array_criteria = [each_char in text for each_char in to_replace]
        if check_box_var and any(array_criteria):
            return(replace_characters(text,replace_by, to_replace, False))
        elif not check_box_var:
            return(replace_characters(text,to_replace, replace_by, True))
        

'''def replace_characters(text,insert: list, remove: list, check_var: bool = False):
        """
            This method replaces a character given by another character given

        Parameters
        ----------
        insert : list
            The list of characters to be removed
        remove : list
            The list of characters to be inserted
        check_var : bool
            Controls which criteria are used, if it is any or none element in the list
        """
        
        if(not check_var):

            index_list = []
            for index, each_char in enumerate(remove):
                index_list.append(text.find(each_char))
            
            lesser_char = [each_index for each_index in index_list if each_index >= 0]
            for index, value in enumerate(lesser_char):
                if value >= 0:
                    text = text.replace(remove[index], insert[index], 1)

        
        return(text)'''
                

def gen_save_file():
    """
        This method saves the generated phrases to a txt file
    """
    with open("output.txt", "w", encoding="utf-8") as output_file:
        output_file.write(last_text)
    



def replace_characters(text,insert: list, remove: list, check_var: bool = False):
            """
                This method replaces a character given by another character given

            Parameters
            ----------
            insert : list
                The list of characters to be removed
            remove : list
                The list of characters to be inserted
            check_var : bool
                Controls which criteria are used, if it is any or none element in the list
            """
            
            if any([each_char in text for each_char in remove]) and \
                    check_var == any([each_char in text for each_char in insert]):
                        index_list = []
                        [index_list.append(text.find(each_char)) for each_char in remove]
                        lesser_char = min([each_index for each_index in index_list if each_index >= 0])
                        lesser_char_index = remove.index(text[lesser_char])
                        text = text.replace(remove[lesser_char_index], insert[lesser_char_index], 1)
            return(text)



"""
    Title: HeadKuarter
    Module Name: passwordGenrator
    Language: Python
    Date Created: 23-05-2022
    Date Modified: 23-05-2022
    Description:
        ###############################################################
        ##  genrate password ussing formosa                          ## 
        ###############################################################
 """

from . import generator
from . import mnemonic
from pathlib import Path
import copy
import random
#import swapper
import json
import gc
import traceback


def get_directory() -> Path:
        """
            This method finds out in which directory the code is running

        Returns
        -------
        path
            Returns the absolute path found of the file
        """
        return Path(__file__).parent.absolute()

# varible 
selected_theme = "finances"
directory_path = (get_directory() / "themes")
files_path = Path(directory_path)
# Finding paths and filtering .json files
themes = tuple([each_directory.stem for each_directory in files_path.glob(r"*.json")])
base_dict = mnemonic.Mnemonic(selected_theme).words_dictionary
edited_dict = copy.deepcopy(base_dict)

global last_text 
global last_n_phrases  
global password_lines
last_text = ""
password_lines = []
last_n_phrases = 0


var_check_number=False
var_check_case = False
var_check_char = False


number_phrases = 1 # initial value

def generate_text(selected_theme,number_phrases,rbn,rbc,sc):
        """
            This method calls object Generator which build a mnemonic phrase in formosa standard
            then updates displayed text
        """
        g = generator.Generator(number_phrases * 32, selected_theme, None)
       
        phrases = g.show_phrases()


        global last_text
        global last_n_phrases 
        global password_lines
        

        text = ""
        phrase_len = len(base_dict["NATURAL_ORDER"])
        for word_index in range(len(phrases)):
            text = text + phrases[word_index][0:2]
        password_genrated = text
        text = text + "\n"
        for phrase_index in range(len(phrases) // phrase_len):
            text = text + " ".join(phrases[phrase_len * phrase_index:phrase_len * (phrase_index + 1)]) + "\n"

        #print(f'bool {rbn}, {rbc}, {sc}')

        insert_number(rbn)
        insert_spc_char(rbc)
        insert_swap_case(sc)

        return(password_genrated)


def insert_number(flag):
        """
            This method changes an ordinary character to a number
        """
        to_replace = ["a", "e", "i", "o", "b", "g", "t"]
        replace_by = ["4", "3", "1", "0", "8", "6", "7"]
        insert_character(to_replace, replace_by, flag)

def insert_spc_char(flag):
        """
            This method changes an ordinary character to a special character
        """
        to_replace = ["c", "h", "l", "s", "j"]
        replace_by = ["¢", "#", "£", "$", "!"]
        insert_character(to_replace, replace_by, flag)

def insert_swap_case(flag):
        """
            This method swaps the case of a character
        """
        to_replace = list(map(chr, range(97, 123)))
        replace_by = [each_char.swapcase() for each_char in to_replace]
        insert_character(to_replace, replace_by, flag)

def insert_character(to_replace: list, replace_by: list, check_box_var: bool):
        """
            This method finds out if the text in the texbox is written as natural or edited
            and calls method replace_characters() with correct order of variables

        Parameters
        ----------
        to_replace : list
            The list of characters to be removed
        replace_by : list
            The list of characters to be inserted
        check_box_var : bool
            This variable tells if the word is written as natural, or it is already edited
        """
        array_criteria = [each_char in last_text for each_char in to_replace]
        if check_box_var and any(array_criteria):
            replace_characters(replace_by, to_replace, False)
        elif not check_box_var:
            replace_characters(to_replace, replace_by, True)
        

def replace_characters(insert: list, remove: list, check_var: bool = False):
        """
            This method replaces a character given by another character given

        Parameters
        ----------
        insert : list
            The list of characters to be removed
        remove : list
            The list of characters to be inserted
        check_var : bool
            Controls which criteria are used, if it is any or none element in the list
        """
        global last_text
        global password_lines
        lines = last_text.splitlines(False)
        for line_index in password_lines:
            # If there is any character to be removed in the line and
            # There is any or none character inserted in the line depending on the control variable "check_var"
            if any([each_char in lines[line_index] for each_char in remove]) and \
                    check_var == any([each_char in lines[line_index] for each_char in insert]):
                index_list = []
                [index_list.append(lines[line_index].find(each_char)) for each_char in remove]
                lesser_char = min([each_index for each_index in index_list if each_index >= 0])
                lesser_char_index = remove.index(lines[line_index][lesser_char])
                new_line = lines[line_index].replace(remove[lesser_char_index], insert[lesser_char_index], 1)
                last_text = last_text.replace(lines[line_index], new_line)
        #print (last_text)        

def gen_save_file():
    """
        This method saves the generated phrases to a txt file
    """
    with open("output.txt", "w", encoding="utf-8") as output_file:
        output_file.write(last_text)
    





"""
    Title: HeadKuarter
    Title: app.py
    Language: Python
    Date Created: 06-05-2022
    Date Modified: 05-05-2022
    Description: CLI Code for linux environment 
        ###############################################################
        ##                     Main file                             ## 
        ###############################################################
 """

from socket import if_nameindex
from typing import Type
from unicodedata import name
import click
from Module.veracrypt import createContainer, mountDrive
from Module.formosa import passwordGenerator
from Module.formosa import passwordGeneratorLC
from Module.gui import rightclick
import getpass
import re
import json
from Module.keepass.keepass import locker,createDb
from os.path import exists as file_exists



@click.group()
def headkurter():
    '''
    A tool to generate password encrypt drive  and Password manager 
    '''
    pass

@click.command()
@click.argument('filename')#, type=click.Path(exists=True)
@click.option('--size', required=True, type=int, help = "Size of disk space in Megabytes")
@click.option('--autopass', is_flag =True, help = "True : Generate Password and display on screen")
@click.option('--encrypt','-encrypt_method',type = click.Choice(['AES', 'Serpent', 'Twofish', 'AES-Twofish', 'AES-Twofish-Serpent','Serpent-AES','Serpent-Twofish-AES','Twofish-Serpent']),default = 'AES')
@click.option('--hash','-hash_methond',type = click.Choice(['RIPEMD-160', 'SHA-512', 'Whirlpool']), default = 'SHA-512')
@click.option('--format','--format-types',type = click.Choice(['exFAT', 'ntfs']), default = 'ntfs')
@click.option('--pwd',type = str, help = "Password to encrypt drive can be Text or Environment Variable")
@click.option('--pim',default = 0)
@click.option('--theme',  type = click.Choice(['finances', 'harry_potter', 'medieval_fantasy', 'the_big_bang_theory','tourism']), default = 'finances', help = "String value from given option")
@click.option('--number', type = click.Choice(['2', '3', '4', '5', '6', '7','8']), default = '2', help = "an integer value between 2-8 ")
@click.option('--num','--insert-number', is_flag =True, help = "When Set insert number in password")
@click.option('--spchar','--insert-spchar', is_flag =True, help = "When Set Insert sepcial character in password")
@click.option('--sc','--swap-case', is_flag =True, help = "When Set change case of password")
@click.option('--cmplx','--complexity', is_flag =True, help = "passsword complexity when True: Generate complex password")
@click.option('--str-pwd','--store-password', type = click.Choice(['display','file','ram']), default = 'file', help = "Option To Store Password")
def create(filename,size,encrypt,hash,format,pim, autopass,theme,number,num,spchar,sc,pwd,cmplx,str_pwd):
    '''
    Format and Create encrypted disk space

    FILENAME  location to creat encrypted drive e.g "/home/username/vctest/<NewFileName>.vc"
    '''
    if autopass:
        number = int(number)
        if(cmplx):
            pwd=passwordGenerator.generate_text(theme,number,num,spchar,sc,str_pwd)
        else:
            pwd=passwordGeneratorLC.generate_text(theme,number,num,spchar,sc,str_pwd)
    elif pwd == None:
        pwd = getpass.getpass()
        pwd2 = getpass.getpass("Confirm Password: ")
        #print (f'pass {pwd}')

        if pwd == "":
            click.echo("either set --auto pass True or provide password --pwd") 
            exit()
        elif pwd != pwd2:
            click.echo("Password and Confirm Password does not match") 
            exit()

    size =f'{str(size)}M' 
    createSuccess = createContainer.container(filename,size,pwd,encrypt,hash,format,pim)
    if (str_pwd == 'display'):
        click.echo(f'Save this password:- {pwd}')
    #click.echo(f'Save this password:- {pwd}')
    #print(createSuccess)




@click.command()
@click.argument('filename')#, type=click.Path(exists=True)
@click.option('--slot',  help = "Optional Slot number to mount encrypted drive")
@click.option('--pwd',type = str, help = "Password to mount encrypted drive. Can be Text or Environment Variable")
@click.option('--pim',default = 0, help = "If you overrode the default PIM when creating your volume, you’ll need to provide it.")
def mount(filename,slot,pwd,pim):
    '''
    Mount Encrypted Drive 

    FILENAME  location of Encrypted Drive e.g "/home/username/vctest/encryptvdisk1.vc"
    '''
    if (pwd == None):
        pwd = getpass.getpass()
    
    mountSuccess = mountDrive.mount_drive(filename,slot,pwd,pim)
    #print(mountSuccess)





@click.command()
@click.argument('filename')#, type=click.Path(exists=True)
@click.option('--slot', help = "Mounted Drive slot ")
def unmount(filename,slot):
    '''
    Unmount Mounted Drive  

    FILENAME  location of Encrypted Drive e.g "/home/username/vctest/encryptvdisk1.vc"
    '''
    if filename != "":
        mountSuccess = mountDrive.unmount(filename)
    elif slot != "":
        mountSuccess = mountDrive.unmount_at(slot)
    #print(mountSuccess)





@click.command()
@click.option('--theme',  type = click.Choice(['finances', 'harry_potter', 'medieval_fantasy', 'the_big_bang_theory','tourism']), default = 'finances', help = "pharase theem that will help in generate password")
@click.option('--number', type = click.Choice(['2', '3', '4', '5', '6', '7','8']), default = '2', help = "an integer value between 2-8 ")
@click.option('--num','--incert-number', is_flag =True, help = "When Set insert number in password")
@click.option('--spchar','--incert-spchar', is_flag =True, help = "When Set Insert sepcial character in password")
@click.option('--sc','--swap-case', is_flag =True, help = "When Set change case of password")
@click.option('--cmplx','--complexity', is_flag =True, help = "passsword complexity")
@click.option('--str-pwd','--store-password', type = click.Choice(['displaye','file','ram']), default = 'file', help = "Option To Store Password")
def generatepassword(theme,number,num,spchar,sc,cmplx,str_pwd):
    '''
    Generate  Password  

    '''
    number = int(number)
    if(cmplx):
        password=passwordGenerator.generate_text(theme,number,num,spchar,sc,str_pwd)
    else:
        password=passwordGeneratorLC.generate_text(theme,number,num,spchar,sc,str_pwd)

    if (str_pwd == 'displaye'):
        click.echo(f'Save this password:- {password}')

@click.command()
@click.argument('locker_name')
@click.option('--pwd',type = str, help = "Password to encrypt locker can be Text or Environment Variable")
def keepassC(locker_name,pwd):
    '''
    Create Locker  

    LOCKER-NAME  locker Name with absolute or relative path e.g "/home/username/vctest/lockername"
    '''
    if pwd == None:
        pwd = getpass.getpass()
        pwd2 = getpass.getpass("Confirm Password: ")
        if pwd != pwd2:
            click.echo("Password and Confirm Password does not match") 
            exit()

    if not locker_name.endswith(".kdbx"):
        locker_name = locker_name+'.kdbx'
    if not file_exists(locker_name):
        createDb(locker_name,pwd)
        print('Locker Space created Succesfully')
    else:
        print('Locker space already exist')

@click.command()
@click.argument('locker_name')
@click.password_option(confirmation_prompt=False)
@click.option('--add_entry', is_flag=True, help ="To add entry in the locker")
@click.option('--group', '-grp', required=False, help ="Group name to creat or find entries inside a particular group" )
@click.option('--dname', '-d', required=False, help ="Domain name for which password to be stored")
@click.option('--uname', '-u', required=False, help ="User name for Domain")
@click.option('--pswd', '-p', required=False, help ="Password for Domain")
@click.option('--find_entries', is_flag=True, help="To find the specific entry in the Locker")
@click.option('--entry', '-ent', required=False)
@click.option('--delete', is_flag=True,help="To delete the specific entry in the locker")
@click.option('--view_pass', is_flag=True,help="To see the password of the specified field")
@click.option('--show', '-show', is_flag=True)
@click.option('--copy', '-copy', is_flag=True, help ="if passed password will be copied to clipboard")
@click.option('--headkurter', is_flag=True, help ="it will activate headkurter prompt to perform multiple operation on logged in locker And to exit type \"exit\"")

def keepassdb(locker_name, password, group, add_entry, dname, uname, pswd, find_entries, entry, delete, view_pass,copy,show,headkurter):
    '''
    Access Locker  

    LOCKER-NAME  locker Name with absolute or relative path e.g "/home/username/vctest/lockername"
    '''
    
    try:
        locker(locker_name,password)
    except:
        click.echo("Incorrect locker-name or password!!")
    else:

        if add_entry:

            if group:
                if not dname:
                    dname = click.prompt("Enter the Domain name")
                if not uname:
                    uname = click.prompt("Enter the username")
                if not pswd:
                    pswd = click.prompt("Enter the password")
                command = [True,True,group,dname,uname,pswd]
                click.echo(locker.createEntryinGrp(self=command))

            else:
                if not dname:
                    dname = click.prompt("Enter the Domain name")
                if not uname:
                    uname = click.prompt("Enter the username")
                if not pswd:
                    pswd = click.prompt("Enter the password")
                command = [True,dname,uname,pswd]
                click.echo(locker.adEntry(self=command))

        elif find_entries:
            if group:
                if entry:
                    command = [True, True, group, entry]
                    click.echo(locker.findEntryInGroup(self=command))
                else:
                    command = [True, True, group]
                    click.echo(locker.findEntryInGroup(self=command))
            elif dname:
                    command = [True, dname]
                    click.echo(locker.findEntries(self=command))
            else:
                command = [True]
                click.echo(locker.findEntries(self=command))
        
        elif delete:
            if group:
                command = [True, True, group]
                click.echo(locker.deleteG(self=command))
            elif dname:
                command = [True, dname]
                click.echo(locker.delete(self=command))
            else:
                c = click.prompt("Do you want to delete a grp (y/n)")
                if c =='Y' or c=='y':
                    group= click.prompt("Enter the group name:- ")
                    command = [True, True, group]
                    click.echo(locker.deleteG(self=command))
                else:
                    dname = click.prompt("Enter the domain you want to delete")
                    command = [True, dname]
                    click.echo(locker.delete(self=command))


        elif view_pass:
            if not dname:
                dname = click.prompt("Enter the Domain name")
            if show:
                command = [True, dname]
                click.echo(locker.retrivepass(self=command))
            if copy:
                command = [True,dname,copy]
                click.echo(locker.retrivepass(self=command))
            else:
                command = [True, dname]
                click.echo(locker.retrivepass(self=command))
        elif headkurter:
            while True:
                command = list(click.prompt('Headkurter~$').strip().replace('-','').split(" "))
                if command[0] == 'add_entry':
                    try:
                        click.echo(locker.adEntry(self=command))
                    except:
                        click.echo("Entry already exists in group")

                elif command[0] == 'find_entries':
                    click.echo(locker.findEntries(self=command))

                elif command[0] == 'delete':
                    click.echo(locker.delete(self=command))

                elif command[0] == 'view_pass':
                    click.echo(locker.retrivepass(self=command))

                elif command[0] == 'exit':
                    click.echo("ThankYou...!!")
                    break
                else:
                    click.echo('%s: command not found' % command[0])

@click.command()

def copycontent():
    '''
    To Enable context menu to copy content of file to clipboard
    '''
    try:
        rightclick.copycontent1()
        click.echo("CopyContent Activated ...!!")
    except:
        click.echo("unable to activate CopyContent some error occurred...!!")
        


@click.command()

def stopcopycontent():
    '''
    To remove copy content from context menu
    '''
    try:
        rightclick.stopcopycontent1()
        click.echo("CopyContent deactivated ...!!")
    except:
        click.echo("unable to deactivate CopyContent some error occurred...!!")
        
    
    
headkurter.add_command(create)
headkurter.add_command(mount)
headkurter.add_command(unmount)
headkurter.add_command(generatepassword)
headkurter.add_command(keepassdb)
headkurter.add_command(keepassC)
headkurter.add_command(copycontent)
headkurter.add_command(stopcopycontent)

if __name__ == "__main__" :
    headkurter()



"""
    Title: HeadKuarter
    Module Name: createContainer
    Language: Python
    Date Created: 7-05-2022
    Date Modified: 7-05-2022
    Description:
        ###############################################################
        ##  Encrypted container created with Veracrypt          ## 
         ###############################################################
 """

import subprocess 
import logging
import json
import os
import sys


creat_at = "C:\\privateVdisk\\encryptvdisk1.vc"
partition_size = "200M"
master_password = "MySuperSecurePassword1!"
encryption_method = "AES"
hash_method = "sha-512"
formate_filesystem= "ntfs"
pim = 0
script = f"\"C:\Program Files\VeraCrypt\VeraCrypt Format.exe\" /create \"{creat_at}\" /size \"{partition_size}\" /password {master_password} /encryption {encryption_method} /hash {hash_method} /filesystem {formate_filesystem} /pim {pim} /silent"

def win_creat(creat_at, partition_size, master_password,encryption_method,hash_method,formate_filesystem, pim):
    '''
       windows Contaner is to create encrypted virtual drive  
    '''

    success = False
    script = f"\"C:\Program Files\VeraCrypt\VeraCrypt Format.exe\" /create \"{creat_at}\" /size \"{partition_size}\" /password {master_password} /encryption {encryption_method} /hash {hash_method} /filesystem {formate_filesystem} /pim {pim} /silent"
    result = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
    #print(result)
    if result.returncode:
            return success
        
    success = True
    return success

def linux_creat(creat_at, partition_size, master_password,encryption_method,hash_method,formate_filesystem, pim):
    '''
       Linux Container is to create encrypted virtual drive  
    '''

    success = False

    script = f"sudo veracrypt --text --create \"{creat_at}\" --size \"{partition_size}\" --password {master_password} --volume-type normal --encryption {encryption_method} --hash {hash_method} --filesystem {formate_filesystem} --pim {pim} --keyfiles \"\" --random-source randomdata.txt"
    result = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)

    #print(result)
    if result.returncode:
            return success
        
    success = True
    return success

def container(creat_at, partition_size, master_password,encryption_method,hash_method,formate_filesystem, pim):
    '''
        Contaner is to create encrypted virtual drive 
    '''

    plt = sys.platform
    if plt=='win32':
        success = win_creat(creat_at, partition_size, master_password,encryption_method,hash_method,formate_filesystem, pim)
    elif plt=='linux':
        success = linux_creat(creat_at, partition_size, master_password,encryption_method,hash_method,formate_filesystem, pim)

    return success



"""
    Title: HeadKuarter
    Module Name: keepaass
    Language: Python
    Date Created: 15-05-2022
    Date Modified: 25-07-2022
    Description:
        ###############################################################
        ##  generate the database using pykeepass to store data      ## 
        ###############################################################
 """
from pykeepass import PyKeePass,create_database


def createDb(locker_name,pwd):
    locker_name = locker_name.split(".")
    fname = locker_name[0]+'.kdbx'
    return create_database (fname, password=pwd, keyfile=None, transformed_key=None)
class locker:
    kdb=''
    def __init__(self,locker_name,pwd):
        self.db = locker_name+'.kdbx'
        self.pwd = pwd
        locker.kdb = PyKeePass(self.db,self.pwd)

    def createEntryinGrp(self):
        length = len(self)
        if length==3:
            grp = self[2]
            gname = input("Enter the domain name:- ")
            uname= input("Enter the username of the desired entry:- ")
            pwd = input("Enter the password:-  ")
        elif length==4:
            grp = self[2]
            gname = self[3]
            uname= input("Enter the username of the desired entry:- ")
            pwd = input("Enter the password:-  ")
        elif length==5:
            grp = self[2]
            gname = self[3]
            uname = self[4]
            pwd = input("Enter the password:-  ")
        elif length==6:
            grp = self[2]
            gname = self[3]
            uname = self[4]
            pwd = self[5]
        else:
            grp = input("Enter the group name:- ")
            gname = input("Enter the Domain name:- ")
            uname= input("Enter the username of the desired entry:- ")
            pwd = input("Enter the password:-  ")
        group =locker.kdb.find_groups(name=grp, first=True)
        if locker.kdb.find_entries(title=gname, first=True):
            return "domain alreay exist"
        if group:
            locker.kdb.add_entry(group, gname, uname, pwd)
        else:
            check = input("Group is not exist! you want to create a new group [y/n]:- ")
            if check=='y' or check=='Y':
                locker.kdb.add_group(locker.kdb.root_group, grp)
                group =locker.kdb.find_groups(name=grp, first=True)
                locker.kdb.add_entry(group, gname, uname, pwd)
            else:
                return "Entry is not added!!"
        locker.kdb.save()
        return "Entry is added successfully!!"

    def adEntry(self):
        length = len(self)
        if '@grp' in self:
            if '@grp' == self[1]:
                return locker.createEntryinGrp(self)
            else: return "@grp is must be at 2nd position"
        elif length>4:
            return "@grp is required at 2nd position if there is 5 argument"
        else:
            if length==2:
                gname = self[1]
                uname = input("Enter the username of the desired entry:- ")
                pwd = input("Enter the password:-  ")
            elif length==3:
                gname = self[1]
                uname = self[2]
                pwd = input("Enter the password:-  ")
            elif length==4:
                gname = self[1]
                uname = self[2]
                pwd = self[3]
            else:
                gname = input("Enter entry doamin name:- ")
                uname= input("Enter the username of the desired entry:- ")
                pwd = input("Enter the password:-  ")
            grp = locker.kdb.find_entries(title=gname, first=True)
            if grp:
                return "Domain already exist!!"
        locker.kdb.add_entry(locker.kdb.root_group, gname, uname, pwd)
        locker.kdb.save()
        return "Entry is added successfully!!"
        
    def retrivepass(self):
        if len(self)>1:
            grp_name = self[1]
        else:
            grp_name = input("Enter the name of Domain:- ")
        grp = locker.kdb.find_entries(title=grp_name, first=True)
        if grp:

            if len(self)>2:
                if self[2]:
                    pyperclip.copy(grp.password)
                    return "Password is copied to clipboard!"
                return "invalid command"
            return grp.password
        return "Domain not found"

    def delete(self):
        length = len(self)
        print(length,self)
        if '@grp' in self:
            if '@grp' == self[1]:
                return locker.deleteG(self)
            else: return "@grp is must be at 2nd position"
        elif length==2:
            ent = self[1]
        else:
            ent = input("Enter the entry you want to delete:- ")
        fnd = locker.kdb.find_entries(title=ent, first=True)
        if fnd:
            locker.kdb.delete_entry(fnd)
            locker.kdb.save()

            return "Entry deleted"
        return "Entry not found"


    def deleteG(self):
        print(len(self),self)

        if len(self)==3:
            group = self[2]
        else:
            group = input("Enter the entry you want to delete:- ")
        grp = locker.kdb.find_groups(name=group, first=True)
        if grp:
            locker.kdb.delete_group(grp)
            locker.kdb.save()

            return "Group deleted"
        return "Group not found"
    def findEntries(self):

        length = len(self)
        if length == 1:
            fnd = locker.kdb.entries
        elif '@grp' == self[1]:
            return locker.findEntryInGroup(self)
        elif length==2:
            name = self[1]
            fnd = locker.kdb.find_entries(title=name, first=True)
        elif length>2:
            return "@grp is required at 2nd position"
        else:
            name = input("Enter the name to find:- ")
            fnd = locker.kdb.find_entries(title=name, first=True)
        if fnd:
            return fnd
        return "Entry not found"

    def findEntryInGroup(self):

        if len(self)==3:
            group = self[2]
            grp = locker.kdb.find_groups(name=group, first=True)
            fnd = grp.entries
        elif len(self)==4:
            group = self[2]
            name = self[3]
            fnd = locker.kdb.find_entries(title=name, first=True)
        else:
            group = input("Enter the group name:")
        if fnd:
            return fnd
        return "Group entry is empty!!"



import subprocess
def copycontent1():
    '''script = f'sudo apt install python3-nutilus'
    naut = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell=True)
    print(naut)'''
    #cop = subprocess.Popen(["cp", "~/CopyContent", "~/.local/share/nautilus/scripts/"])
    script = f'cp Module/gui/CopyContent ~/.local/share/nautilus/scripts/'
    cop = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell=True)
    #print(cop)

    #cop.wait()
    script = 'chmod u+x ~/.local/share/nautilus/scripts/CopyContent'
    excu = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell=True)
    #print(excu)

def stopcopycontent1():
    
    cop = subprocess.run("rm ~/.local/share/nautilus/scripts/CopyContent", stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell= True)
    
    


#
# Copyright (c) 2013 Pavol Rusnak
# Copyright (c) 2017 mruddy
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is furnished to do
# so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

import os
import hashlib
import hmac
import itertools
import json
from pathlib import Path
from typing import AnyStr, List, Sequence, TypeVar, Union
import unicodedata


_T = TypeVar("_T")
PBKDF2_ROUNDS = 2048


class ConfigurationError(Exception):
    pass


# Refactored code segments from <https://github.com/keis/base58>
def b58encode(v: bytes) -> str:
    alphabet = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"

    p, acc = 1, 0
    for c in reversed(v):
        acc += p * c
        p = p << 8

    string = ""
    while acc:
        acc, idx = divmod(acc, 58)
        string = alphabet[idx: idx + 1] + string
    return string


def verify_file(words_dictionary: dict):
    """
        This function does the complete check in the lists and keys of the dictionary according to the Formosa standard

    Parameters
    ----------
    words_dictionary :
        This is the base dictionary containing all lists of words in Formosa standard
    """
    for filling_order_word in words_dictionary["FILLING_ORDER"]:

        current_dict = words_dictionary[filling_order_word]
        # Test if the sequence of restrictions are consistent
        if not words_dictionary[filling_order_word]["RESTRICTED_BY"] == "NONE":
            if not (
                (words_dictionary["FILLING_ORDER"].index(current_dict["RESTRICTED_BY"]) <
                 words_dictionary["FILLING_ORDER"].index(filling_order_word))
            ):
                raise ConfigurationError(
                    "List sequence inconsistent restriction order for %s. (restricted_by checking)."
                    % filling_order_word
                )
        # For every syntactic function restricted by current one
        for restricted_syntax1 in current_dict["RESTRICTS"]:
            for image_word in current_dict[restricted_syntax1]["IMAGE"]:
                for restricted_syntax2 in words_dictionary[restricted_syntax1]["RESTRICTS"]:
                    if not (image_word in list(words_dictionary[restricted_syntax1][restricted_syntax2]["MAPPING"])):
                        raise ConfigurationError(
                            "The word \"%s\" from the image list of \"%s\" is not contained in the keys list"
                            % (image_word, restricted_syntax1)
                        )

            for key in list(current_dict[restricted_syntax1]["MAPPING"]):
                if not (key in current_dict["TOTAL_LIST"]):
                    raise ConfigurationError(
                        "List of keys should be the same, but it is not, "
                        "the key \"%s\" in dictionary of \"%s\" in \"%s\" keys."
                        % (key, filling_order_word, restricted_syntax1)
                    )
            if not (words_dictionary["FILLING_ORDER"].index(restricted_syntax1) >
                    words_dictionary["FILLING_ORDER"].index(filling_order_word)):
                raise ConfigurationError(
                    "List sequence inconsistent restriction order for %s. (restricts checking)."
                    % restricted_syntax1
                )
            # Check there are enough sublists
            if not len(current_dict[restricted_syntax1]["MAPPING"]) >= 2 ** current_dict["BIT_LENGTH"]:
                raise ConfigurationError(
                    "Wordlist of %s should contain %d key words in %s list, but it contains %d key words."
                    % (filling_order_word, 2 ** current_dict["BIT_LENGTH"],
                       restricted_syntax1, len(current_dict[restricted_syntax1]["MAPPING"]))
                )
            # For each syntactic function restricted by current one
            line_bits_length = 2 ** words_dictionary[restricted_syntax1]["BIT_LENGTH"]
            # For each line in the sublist
            for line_key in current_dict[restricted_syntax1]["MAPPING"]:
                list_length = (len(current_dict[restricted_syntax1]["MAPPING"][line_key]))
                # Check whether length has correct value
                if (
                    list_length != line_bits_length
                ):
                    raise ConfigurationError(
                        "Key \"%s\", in \"%s\" restriction, should contain %d words, but it contains %d words."
                        % (line_key, restricted_syntax1, line_bits_length, list_length)
                    )
                for word in current_dict[restricted_syntax1]["MAPPING"][line_key]:
                    if not(word in words_dictionary[restricted_syntax1]["TOTAL_LIST"]):
                        raise ConfigurationError(
                            "Word \"%s\" not found in list of total words in \"%s\"."
                            % (word, restricted_syntax1)
                        )


def concat_idx_of_words(
    passphrase: list,
    concat_bits: list,
    words_dictionary: dict,
    phrase_len: int
) -> list:
    """
        This function maps each word to each index in the restriction list in form of concatenated bits

    Parameters
    ----------
    passphrase : list
        This is the list of words used in Formosa standard
    concat_bits :list
        This is an empty list to be returned as concatenated bits
    words_dictionary : dict
        This is the base dictionary from which the word lists are consulted
    phrase_len : int
        This is the size of phrase used in the Formosa standard

    Returns
    -------
    list
        Returns the concatenated bits mapped from the passphrase
    """
    bit_idx = 0
    restricting_word = ""

    for sentence_idx in range(len(passphrase) // phrase_len):

        current_sentence = passphrase[phrase_len * sentence_idx: phrase_len * (sentence_idx + 1)]

        for syntactic_key in words_dictionary["FILLING_ORDER"]:
            restricted_by = words_dictionary[syntactic_key]["RESTRICTED_BY"]
            natural_word_position = words_dictionary["NATURAL_ORDER"].index(syntactic_key)
            len_word = words_dictionary[syntactic_key]["BIT_LENGTH"]
            current_word = current_sentence[natural_word_position]
            if restricted_by != "NONE":
                restricting_idx = words_dictionary["NATURAL_ORDER"].index(restricted_by)
                restricting_word = current_sentence[restricting_idx]

            wdict_idx = (
                words_dictionary[syntactic_key]["TOTAL_LIST"].index(current_word)
                if words_dictionary[syntactic_key]["RESTRICTED_BY"] == "NONE" else
                words_dictionary[restricted_by][syntactic_key]["MAPPING"][restricting_word].index(current_word)
            )
            if wdict_idx == -1:
                raise LookupError("Unable to find \"%s\" in the \"%s\" word list." % (current_word, restricted_by))

            for wbit_idx in range(len_word):
                concat_bits[bit_idx] = (wdict_idx & (1 << (len_word - 1 - wbit_idx))) != 0
                bit_idx += 1
    return concat_bits


class Mnemonic(object):
    def __init__(self, theme: str):
        
        with open(
            (self._get_directory() / "themes" / ("%s.json" % theme))
        ) as json_file:
            
            self.words_dictionary = json.load(json_file)
            

        verify_file(self.words_dictionary)

    @staticmethod
    def _get_directory() -> Path:
        """
            This method finds out in which directory the code is running

        Returns
        -------
        path
            Returns the absolute path found of the file
        """
        return Path(__file__).parent.absolute()

    @classmethod
    def list_languages(cls) -> List[str]:
        return [
            f.split(".")[0]
            for f in os.listdir(cls._get_directory())
            if f.endswith(".txt")
        ]

    @staticmethod
    def normalize_string(txt: AnyStr) -> str:
        if isinstance(txt, bytes):
            utxt = txt.decode("utf8")
        elif isinstance(txt, str):
            utxt = txt
        else:
            raise TypeError("String value expected")

        return unicodedata.normalize("NFKD", utxt)

    @classmethod
    def detect_language(cls, code: str) -> str:
        #        ------------------------hardcoded------------------------
        #         Returns 'enslish' for temporary list of words
        return "english"
        
#        code = cls.normalize_string(code)
#        first = code.split(" ")[0]
#        languages = cls.list_languages()
#        print(languages)
#
#        for lang in languages:
#            mnemo = cls(lang)
#            if first in mnemo.wordlist:
#                return lang
#
#        raise ConfigurationError("Language not detected")
#        ------------------------hardcoded------------------------

    def generate(self, strength: int = 128) -> str:
        if strength not in range(128, 257, 32):  # [128, 160, 192, 224, 256]:
            raise ValueError(
                "Strength should be one of the following [128, 160, 192, 224, 256], but it is %d."
                % strength
            )
        return self.to_mnemonic(os.urandom(strength // 8))

    # Adapted from <http://tinyurl.com/oxmn476>
    def to_entropy(self, passphrase: Union[List[str], str]) -> bytearray:
        """
            This method extract an entropy and checksum values from passphrase in Formosa standard

        Parameters
        ----------
        passphrase : list or str
            This is the passphrase that is desired to extract entropy from

        Returns
        -------
        bytearray
            Returns a bytearray with the entropy and checksum values extracted from a passphrase in a Formosa standard
        """
        if not isinstance(passphrase, list):
            passphrase = passphrase.split(" ")
        if len(passphrase) not in range(6, 55, 6):
            raise ValueError(
                "Number of words must be one of the following:"
                " 6, 12, 18, 24, 30, 36, 42, 48 or 54, but it is \"%d\"."
                % (len(passphrase))
            )
        phrase_len = len(self.words_dictionary["FILLING_ORDER"])
        # Look up all the words in the list and construct the
        # concatenation of the original entropy and the checksum.

        # Determining strength of password
        entropy_length_bits = len(passphrase) * 32 // 6
        checksum_length_bits = entropy_length_bits // 32
        concat_len_bits = checksum_length_bits + entropy_length_bits

        concat_bits = [False] * concat_len_bits
        concat_bits = (concat_idx_of_words(
            passphrase, concat_bits, self.words_dictionary, phrase_len
        )
        )

        # Extract original entropy as bytes.
        entropy = bytearray(entropy_length_bits // 8)

        # For every entropy byte
        for entropy_idx in range(len(entropy)):
            # For every entropy bit
            for bit_idx in range(8):
                bit_int = 1 if concat_bits[(entropy_idx * 8) + bit_idx] else 0
                entropy[entropy_idx] |= bit_int << (8 - 1 - bit_idx)
        hash_bytes = hashlib.sha256(entropy).digest()
        hash_bits = list(
            itertools.chain.from_iterable(
                [checksum_byte & (1 << (8 - 1 - bit_idx)) != 0
                 for bit_idx in range(8)
                 ] for checksum_byte in hash_bytes
            )
        )

        # Test checksum
        valid = True
        for bit_idx in range(checksum_length_bits):
            valid &= concat_bits[entropy_length_bits + bit_idx] == hash_bits[bit_idx]
        if not valid:
            raise ValueError("Failed checksum.")

        return entropy

    def to_mnemonic(self, data: bytes) -> str:
        """
            This method creates a passphrase in Formosa standard from an entropy and checksum values

        Parameters
        ----------
        data : bytes
            This is the entropy and checksum that is desired to build passphrase from

        Returns
        -------
        str
            Returns a passphrase in a Formosa standard built from a bytes with the entropy and checksum values
        """
        if len(data) not in range(4, 45, 4):
            raise ValueError(
                "Number of phrases should be 1 to 11, but it is \"%d\"."
                % (len(data)//4)
            )
        h = hashlib.sha256(data).hexdigest()
        # Concatenation of string
        b = (
            # String of entropy bits
            bin(int.from_bytes(data, byteorder="big"))[2:].zfill(len(data) * 8)
            # String of checksum bits
            + bin(int(h, 16))[2:].zfill(256)[: len(data) * 8 // 32]
        )

        bit_idx = 0
        ret_list = []
        last_word = ""
        for phrase_idx in range(len(b) // 33):
            current_sentence = ["", "", "", "", "", ""]
            for syntactic_key in self.words_dictionary["FILLING_ORDER"]:
                restricted_by = self.words_dictionary[syntactic_key]["RESTRICTED_BY"]
                syntactic_order = self.words_dictionary["NATURAL_ORDER"].index(syntactic_key)
                len_word = self.words_dictionary[syntactic_key]["BIT_LENGTH"]

                if restricted_by != "NONE":
                    last_idx = self.words_dictionary["NATURAL_ORDER"].index(restricted_by)
                    last_word = current_sentence[last_idx]

                list_of_words = (
                    self.words_dictionary[syntactic_key]["TOTAL_LIST"]
                    if self.words_dictionary[syntactic_key]["RESTRICTED_BY"] == "NONE"
                    else self.words_dictionary[restricted_by][syntactic_key]["MAPPING"][last_word]
                )

                # Integer from substring of zeroes and ones representing index of current word within its list
                wdict_idx = int(b[bit_idx: bit_idx + len_word], 2)
                bit_idx += len_word
                current_sentence[syntactic_order] = list_of_words[wdict_idx]

            ret_list += current_sentence
        ret = " ".join(ret_list)
#        ------------------------hardcoded------------------------
#        if (
#            self.detect_language(" ".join(result)) == "japanese"
#        ):  # Japanese must be joined by ideographic space.
#            result_phrase = u"\u3000".join(result)
#        else:
#            result_phrase = " ".join(result)
#        result_phrase = " ".join(result)
#        ------------------------hardcoded------------------------
        return ret

#    ------------------------hardcoded------------------------
    def check_item(self, mnemonic: str) -> bool:
        mnemonic_list = self.normalize_string(mnemonic).split(" ")
        # list of valid mnemonic lengths
        if len(mnemonic_list) not in range(12, 25, 3):
            return False
        try:
            idx = map(
                lambda x: bin(self.wordlist.index(x))[2:].zfill(11), mnemonic_list
            )
            b = "".join(idx)
        except ValueError:
            return False
        l = len(b)  # noqa: E741
        d = b[: l // 33 * 32]
        h = b[-l // 33:]
        nd = int(d, 2).to_bytes(l // 33 * 4, byteorder="big")
        nh = bin(int(hashlib.sha256(nd).hexdigest(), 16))[2:].zfill(256)[: l // 33]
        return h == nh
#       ------------------------hardcoded------------------------

    def expand_word(self, prefix: str) -> str:
        for i in range(4):
            self.wlist = str(self.wordlist[i][:])
            if prefix in self.wlist:
                return prefix
            else:
                matches = [word for word in self.wlist if word.startswith(prefix)]
                if len(matches) == 1:  # matched exactly one word in the wordlist
                    return matches[0]
                else:
                    # exact match not found.
                    # this is not a validation routine, just return the input
                    return prefix
#       ------------------------hardcoded------------------------

    def expand(self, mnemonic: str) -> str:
        return " ".join(map(self.expand_word, mnemonic.split(" ")))
#   ------------------------hardcoded------------------------

    @classmethod
    def to_seed(cls, mnemonic: str, passphrase: str = "") -> bytes:
        mnemonic = cls.normalize_string(mnemonic)
        passphrase = cls.normalize_string(passphrase)
        passphrase = "mnemonic" + passphrase
        mnemonic_bytes = mnemonic.encode("utf-8")
        passphrase_bytes = passphrase.encode("utf-8")
        stretched = hashlib.pbkdf2_hmac(
            "sha512", mnemonic_bytes, passphrase_bytes, PBKDF2_ROUNDS
        )
        return stretched[:64]

    @staticmethod
    def to_hd_master_key(seed: bytes, testnet: bool = False) -> str:
        if len(seed) != 64:
            raise ValueError("Provided seed should have length of 64")

        # Compute HMAC-SHA512 of seed
        seed = hmac.new(b"Bitcoin seed", seed, digestmod=hashlib.sha512).digest()

        # Serialization format can be found at:
        # https://github.com/bitcoin/bips/blob/master/bip-0032.mediawiki#Serialization_format
        xprv = b"\x04\x88\xad\xe4"  # Version for private mainnet
        if testnet:
            xprv = b"\x04\x35\x83\x94"  # Version for private testnet
        xprv += b"\x00" * 9  # Depth, parent fingerprint, and child number
        xprv += seed[32:]  # Chain code
        xprv += b"\x00" + seed[:32]  # Master key

        # Double hash using SHA256
        hashed_xprv = hashlib.sha256(xprv).digest()
        hashed_xprv = hashlib.sha256(hashed_xprv).digest()

        # Append 4 bytes of checksum
        xprv += hashed_xprv[:4]

        # Return base58
        return b58encode(xprv)


def main() -> None:
    import sys

    if len(sys.argv) > 1:
        hex_data = sys.argv[1]
    else:
        hex_data = sys.stdin.readline().strip()
    data = bytes.fromhex(hex_data)
    m = Mnemonic("english")
    print(m.to_mnemonic(data))


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
import app

setup(
    name='headkuter',
    version=1.5,
    packages = ['Module','Module.veracrypt','Module.keepass','Module.formosa','Module.gui','Module.formosa.themes'],
    py_modules=['app','autoVera'],
    package_data={'Module.formosa.themes':['finances.json','copy_left.json','harry_potter.json','medieval_fantasy.json','sci-fi.json','the_big_bang_theory.json','tourism.json']},
    entry_points={
        'console_scripts': [
            'headkuter =app:headkurter'  #ponting to executeable function
        ]
    },
    install_requires=[
        'click==8.1.3',
        'colorama==0.4.4',
        'pykeepass==4.0.3',
        'pyperclip==1.8.2',

    ],
    include_package_data= True,
        #zip_safe = False
    )


"""
    Title: HeadKuarter
    Module Name: mountDrive
    Language: Python
    Date Created: 7-05-2022
    Date Modified: 7-05-2022
    Description:
        ###############################################################
        ##  mounting Veracrypt Encrypted container                   ## 
        ###############################################################
 """

import subprocess 
import logging
import json
import os

creat_at = "C:\\privateVdisk\\encryptvdisk1.vc"
drive = "X" # any letter
master_password = "MySuperSecurePassword1!"

def mount_drive(creat_at,slot,master_password,pim):
    
    '''
    mount the encryted file to the drive
    '''
    success = False
    
    if slot != None:
        script = f"sudo veracrypt --text --mount {creat_at} /mnt --password {master_password} --pim {pim} --keyfiles \"\" --protect-hidden no --slot {slot} --verbose"
    else :
        script = f"sudo veracrypt --text --mount {creat_at} /mnt --password {master_password} --pim {pim} --keyfiles \"\" --protect-hidden no --verbose"
        
    result = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell=True)
    #print(result)
    if result.returncode:
            return success
        
    success = True
    return success

def unmount(creat_at):
    #unmount encrypted drive
    success = False
    script = f"sudo veracrypt --text --dismount \"{creat_at}\""
    result = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell=True)
    #print(result)
    if result.returncode:
            return success
        
    success = True
    return success

def unmount_at(drive):
    #unmount encrypted drive
    success = False
    script = f"sudo veracrypt --text --dismount {drive} --verbose "
    result = subprocess.run(script,stdout=subprocess.PIPE, stderr=subprocess.STDOUT,shell=True)
    #print(result)
    if result.returncode:
            return success
        
    success = True
    return success